#include "sed1330.h"

// *******************************************************
// Function Prototypes 
void lcdSendData   (unsigned  char value );            /* Send Data Byte                 */
void lcdSendCommand(unsigned  char value );            /* Send Command Byte              */
void lcdReadData   (unsigned char *);                  /* Get Data Byte                  */
void lcdTextCursor (unsigned char x, unsigned char y); /* Set Text Cursor                */
void lcdGraphCursor(unsigned char x, unsigned char y); /* Set Graphic Cursor             */
void lcdScrollUp   (void);                             /* Scroll Text Screen up one line */
//*******************************************************
// defines
//*******************************************************

// *************************************************************
// Sed 1330 Command Set 
// *************************************************************
#define SYSSET	      0x40    /* System Set                              */
#define SLEEPIN	      0x53    /* Enter standby mode                      */
#define DISPOFF       0x58    /* Display on                              */
#define DISPON        0x59    /* Display off                             */
#define SCROLL        0x44    /* Display Start Addr & Display regions    */
#define CSRFORM       0x5D    /* Set Cursor Type                         */
#define CGRAM         0x5C    /* Start Addr Char. Generator Ram          */
#define CSRDIR        0x4C    /* Direction of cursor movement            */
#define HDOTSCR       0x5A    /* Horizontal Scroll Position              */
#define OVERLAY       0x5B    /* Display Overlay format                  */
#define CSRW          0x46    /* Set Cursor Address                      */
#define CSRR          0x47    /* Read Cursor Address                     */
#define MWRITE        0x42    /* Write to Display Memory                 */
#define MREAD         0x43    /* Read from Display Memory                */



#define short_delay ({          \
        asm volatile (          \
                "nop" "\n\t"    \
                "nop" "\n\t"    \
                "nop" "\n\t"    \
                "nop" "\n\t"    \
                "nop" "\n\t"    \
                "nop" "\n\t"    \
                "nop"           \
                );              \
        })

static unsigned char currTextx = 0;
static unsigned char currTexty = 0;

/*********************************************************/
/* send Command to Controler                             */
/*********************************************************/
void lcdSendCommand(unsigned  char command )
{

    GLCD_CTRL |=  _BV(GLCD_RD);  //Disablr read
    GLCD_DATA  = command;        //Load command
    GLCD_CTRL |=  _BV(GLCD_COMMAND);  //A0 = 1;
    GLCD_CTRL &= ~_BV(GLCD_CS);       //Enable CS;
    GLCD_CTRL &= ~_BV(GLCD_WR);       //Enable Write signal
    short_delay;
    GLCD_CTRL |=  _BV(GLCD_CS);
    GLCD_CTRL |=  _BV(GLCD_WR);
    GLCD_CTRL &= ~_BV(GLCD_COMMAND);
     
}


/*********************************************************/
/* send Data to Controller                               */
/*********************************************************/
void lcdSendData(unsigned char value )
{ 
    GLCD_DATA    = value;

    GLCD_CTRL &= ~_BV(GLCD_COMMAND);   //A0 = 0;
    GLCD_CTRL |=  _BV(GLCD_RD);        //No reading
    GLCD_CTRL &= ~_BV(GLCD_CS);        //Enable CS
    GLCD_CTRL &= ~_BV(GLCD_WR);				//Enable Write command
    short_delay;    
    GLCD_CTRL |=  _BV(GLCD_CS);
    GLCD_CTRL |=  _BV(GLCD_WR);
}

/*********************************************************/
/* read Data from Controller                               */
/*********************************************************/
void lcdReadData(unsigned char *theByte)
{
    lcdSendCommand(MREAD);  /* Read Data                 */
    DDR_DATA  =   0x00;     /* Set pins for Input */

    GLCD_CTRL |=  _BV(GLCD_COMMAND);
    GLCD_DATA    = 0xFF;
    GLCD_CTRL |=  _BV(GLCD_WR);
    GLCD_CTRL &= ~_BV(GLCD_CS);
    GLCD_DATA    = 0xFF;
    
    GLCD_CTRL &= ~_BV(GLCD_RD);
    short_delay;
    short_delay;
    *theByte      = PIN_DATA;
    GLCD_CTRL |=  _BV(GLCD_CS);
    GLCD_CTRL |=  _BV(GLCD_RD);

    GLCD_DATA =   0x00;
    DDR_DATA  =   0xFF; /* Set for Output */


}



/*********************************************************/
/* Reset Controller                                      */
/*********************************************************/
void GLCD_Reset(void)
{
int i;
volatile int j; 

    DDR_DATA  = 0xFF;
    DDR_CTRL  = 0xFF;

    GLCD_CTRL |=  _BV(GLCD_RD);
    GLCD_CTRL |=  _BV(GLCD_WR);
    GLCD_DATA    = 255;
    GLCD_CTRL |=  _BV(GLCD_COMMAND);

    GLCD_CTRL &= ~_BV(GLCD_RESET);
    for (i = 0; i < 1000; i++)
    {  j++;
       GLCD_CTRL &= ~_BV(GLCD_RESET);
    }
    for (i = 0; i < 1000; i++)
    {  j++;
       GLCD_CTRL |=  _BV(GLCD_RESET);
    }
       
    GLCD_CTRL |=  _BV(GLCD_RESET);
    GLCD_CTRL |=  _BV(GLCD_CS);

    
}

/*********************************************************/
/* Init Controller                                       */
/* Here we have a couple of param hardcoded
   needs to be changed to calculated values
   in future releases.
   Sorry - up to then you have to check maually
   
*/
/*********************************************************/
void GLCD_Init(void)
{
  DDR_DATA  = 0xFF;  //Set data port as output
  DDR_CTRL  = 0xFF;  //Set ctrl port as output

	GLCD_Reset();


  lcdSendCommand(SYSSET);
  lcdSendData(0x30);            //lcdSendData(0x30);
  lcdSendData((GLCD_CHARWIDTH-1)|0x80); /* Char width              */
  lcdSendData(GLCD_CHARHEIGHT-1);    /* Char height             */
  lcdSendData(GLCD_CHARPERROW-1);    /* Chars per row           */
  lcdSendData(GLCD_CHARPERROW+4);    /* Frame Frequency control */
  lcdSendData(GLCD_RESY-1);          /* Lines per Graphic Screen*/
  lcdSendData(GLCD_CHARPERROW);      /* Virtual Screen Width low byte    */
  lcdSendData(0);                   /* Virtual Screen Width high byte   */
  
  lcdSendCommand(OVERLAY);
  lcdSendData(1);                   /* XOR Simple Overlay was 1*/
  
  lcdSendCommand(SCROLL);
  lcdSendData(0);             /* Low  Byte Start Text Mem  */
  lcdSendData(0);             /* High Byte Start Text Mem  */
  lcdSendData(GLCD_RESY-1);    /* Screen Lines              */
  lcdSendData(0x80);          /* Low Byte Start Graph Mem  */
  lcdSendData(0x02);          /* High Byte Start Graph Mem */
  lcdSendData(GLCD_RESY-1);    /* Screen Lines              */
  
  lcdSendCommand(CSRFORM);
  lcdSendData(0x05);          /* Cursor width   */
  lcdSendData(0x85);          /* Cursor height High Bit = BLock Cursor */

  lcdSendCommand(HDOTSCR);     /* Display on*/
  lcdSendData(0x0);            /* was 16Cursor FLash, first+second screen block on  */
  

  lcdSendCommand(CSRDIR);     /* No Param = shift right */
  
  lcdSendCommand(DISPON);     /* Display on*/
  lcdSendData(0x14);          /* Do not switch on Cursor ( messes up Graphics ) */
  currTextx=0;
  currTexty=0;
 
}


/*********************************************************/
/* Set Text Cursor                                       */
/* 0,0 to LCDCHARPERROW-1, GLCD_RESY/GLCD_CHARHEIGHT     */
/* 0,0 to 39,15                                          */
/*********************************************************/
void lcdTextCursor(unsigned char x, unsigned char y)
{
  int csraddr;


  csraddr = 0x00 + (y*GLCD_CHARPERROW) + x;
  lcdSendCommand(CSRW);               /* Set Cursor */
  lcdSendData((unsigned char)(csraddr & 0xff));           /* Cursor low  byte */
  lcdSendData((unsigned char)( (csraddr >> 8 ) & 0xff) ); /* Cursor high byte */
  currTextx=x;
  currTexty=y;

}

/*********************************************************/
/* Set Text Cursor                                       */
/* 0,0 to LCDCHARPERROW-1, GLCD_RESY/GLCD_CHARHEIGHT     */
/* 0,0 to 39,15                                          */
/*********************************************************/
void lcdTextCursorInternal(unsigned char x, unsigned char y)
{
  int csraddr;


  csraddr = 0x00 + (y*GLCD_CHARPERROW) + x;
  lcdSendCommand(CSRW);               /* Set Cursor */
  lcdSendData((unsigned char)(csraddr & 0xff));           /* Cursor low  byte */
  lcdSendData((unsigned char)( (csraddr >> 8 ) & 0xff) ); /* Cursor high byte */

}



/*********************************************************/
/* Set Graphic Cursor                                    */
/* 0,0 to GLCD_RESX-1, GLCD_RESY-1                         */
/* 0,0 to 239,127                                        */
/*********************************************************/
void lcdGraphCursor(unsigned char myx, unsigned char y)
{
  int csraddr;
  
  // Here the base addr of graphic is still FIXED
  csraddr = 0x0280 + (y*GLCD_CHARPERROW);
  csraddr += ((unsigned char)(myx / 6));
  lcdSendCommand(CSRW);               /* Set Cursor */
  lcdSendData((unsigned char)(csraddr & 0xff));           /* Cursor low  byte */
  lcdSendData((unsigned char)( (csraddr >> 8 ) & 0xff) ); /* Cursor high byte */
  
}



/*********************************************************/
/* Text Clear Screen                                     */
/*********************************************************/
void GLCD_ClearText(void)
{
int i;
  lcdTextCursor(0,0);
  lcdSendCommand(MWRITE);           
  for (i = 0; i < (GLCD_CHARPERROW*GLCD_CHARROWS); i++)
       lcdSendData(0x00);
  lcdTextCursor(0,0);
}

 
/*********************************************************/
/* Graphic Clear Screen                                     */
/*********************************************************/
void GLCD_ClearGraph(void)
{
  int i,j;
  i = 0;
  lcdGraphCursor(0,0);
  lcdSendCommand(MWRITE);           
  for (j = 0; j < GLCD_RESY; j++)
  for (i = 0; i < (GLCD_RESX / GLCD_CHARWIDTH); i++)
       lcdSendData(0x00);
  lcdTextCursor(0,0);

}


/*********************************************************/
/* Scroll Screen up one line                             */
/*********************************************************/

void lcdScrollUp(void)
{
unsigned char zeile,spalte;
unsigned char c;

  for (zeile=1; zeile< GLCD_CHARROWS  ; zeile++)
  {
    for (spalte=0;spalte < GLCD_CHARPERROW; spalte++)
    {
       lcdTextCursorInternal(spalte, zeile);
       lcdReadData(&c);                /* Get the byte from Display */
       lcdTextCursorInternal(spalte, zeile-1);
       lcdSendCommand(MWRITE);           
       lcdSendData(c);
    }
  }  
  for (spalte=0;spalte < GLCD_CHARPERROW; spalte++)
  {
     lcdSendCommand(MWRITE);           
     lcdSendData(' ');
  }
  lcdTextCursorInternal(currTextx, currTexty); /* Back to Text Mode at old Position */          
}

/*********************************************************/
/* Print Text                                            */
/*********************************************************/
void GLCD_Text(unsigned char x, unsigned char y, unsigned char *myText)
{  
  lcdTextCursor(x,y);
  lcdSendCommand(MWRITE);           
  while (*myText)
  {
  	lcdSendData(*myText);
  	myText++;
  	currTextx++;
  	if (currTextx >= GLCD_CHARPERROW)
  	{
  	   currTextx = 0;
  	   currTexty++;
  	   if (currTexty >= GLCD_CHARROWS)
  	   {
              currTexty--;
              lcdScrollUp ();
           }
  	}
  }
}


/*********************************************************/
/* Print Char                                            */
/*********************************************************/
void GLCD_Char(unsigned char myChar)
{  
  if ( myChar < ' ')
  {
    switch (myChar)
    {
         case 12: // FormFeed
          GLCD_ClearText();
          break;
         case '\r': // CR
          currTextx = 0;
          lcdTextCursor(0, currTexty);
          break;
         case '\n': // LF
          currTexty++;
          if (currTexty >= GLCD_CHARROWS)
          {
             currTexty--;
             lcdScrollUp ();
          }
          lcdTextCursor(currTextx, currTexty);
          break;
         case 0x08: // BS
          if (currTextx > 0)
            currTextx --; 
          lcdTextCursor(currTextx, currTexty);
          break;
    }
  }
  else
  {
    lcdSendCommand(MWRITE);           
    lcdSendData(myChar);
    currTextx++;
    if (currTextx >= GLCD_CHARPERROW)
    {
       currTextx = 0;
       currTexty ++;
    }
    if (currTexty >= GLCD_CHARROWS)
    {
       currTexty--;
       lcdScrollUp ();
    }
  }
}


/*********************************************************/
/* Set/Reset Pixel                                       */
/*********************************************************/
void GLCD_Pixel(int x,int y, drawmode show)
{
unsigned char myByte;
unsigned char tmp;
  
  lcdGraphCursor(x, y);                /* Set Gra Cursor            */
  lcdReadData(&myByte);                /* Get the byte from Display */
  lcdGraphCursor(x, y);                /* Set Gra Cursor            */
  lcdSendCommand(MWRITE);              /* Write Data                */

  if ( (show == on) || (show == fill) ) 
  {
    //tmp = (unsigned char)~( 0x80 >> (int)(x%6));
    //myByte  &= tmp ;
    tmp = (unsigned char)( 0x80 >> (int)(x%6));
    myByte  |= tmp ;
  }
  else
  {
    //tmp = (unsigned char)( 0x80 >> (int)(x%6));
    //myByte  |= tmp ;
    tmp = (unsigned char)~( 0x80 >> (int)(x%6));
    myByte  &= tmp ;
  }
  
  myByte   |= 0x3;
  lcdSendData(myByte);
  lcdTextCursorInternal(currTextx, currTexty); /* Back to Text Mode at old Position */
}


/****************************************************************************/
/* Draws a rectangle from x1,y1 to x2,y2.                                   */
/****************************************************************************/
void GLCD_Rectangle(int x1,int y1,int x2,int y2,drawmode show)
{
  int i;
  for (i=x1; i<=x2; i++) GLCD_Pixel(i,y1, show);
  for (i=x1; i<=x2; i++) GLCD_Pixel(i,y2, show); 
  for (i=y1; i<=y2; i++) GLCD_Pixel(x1,i, show);
  for (i=y1; i<=y2; i++) GLCD_Pixel(x2,i, show);
  if ( (show == fill)||(show == clear) )
  {
      for (i=y1; i<=y2; i++) GLCD_Line(x1,i,x2,i, show);
  } 
}

/****************************************************************************/
/* Draws a line from x,y at given degree from inner_radius to outer_radius. */
/****************************************************************************/
void GLCD_DegreeLine(int x,int y, int degree, int inner_radius, int outer_radius, drawmode show)
{
  int fx,fy,tx,ty;
  fx = x + (inner_radius * sin(degree * 3.14 / 180));
  fy = y - (inner_radius * cos(degree * 3.14 / 180));
  tx = x + (outer_radius * sin(degree * 3.14 / 180));
  ty = y - (outer_radius * cos(degree * 3.14 / 180));
  GLCD_Line(fx,fy,tx,ty,show);
}

/****************************************************************************/
/* Draws a circle with center at x,y with given radius.                     */
/* Set show to 1 to draw pixel, set to 0 to hide pixel.                     */
/****************************************************************************/
void GLCD_Circle(int x, int y, int radius, drawmode  show)
{
  int xc = 0;
  int yc = radius;
  int p = 3 - (radius<<1);
  while (xc <= yc)  
  {
    GLCD_Pixel(x + xc, y + yc, show);
    GLCD_Pixel(x + xc, y - yc, show);
    GLCD_Pixel(x - xc, y + yc, show);
    GLCD_Pixel(x - xc, y - yc, show);
    GLCD_Pixel(x + yc, y + xc, show);
    GLCD_Pixel(x + yc, y - xc, show);
    GLCD_Pixel(x - yc, y + xc, show);
    GLCD_Pixel(x - yc, y - xc, show);
    if (p < 0)
      p += (xc++ << 2) + 6;
    else
      p += ((xc++ - yc--)<<2) + 10;
  }
}


/****************************************************************************/
/* Draws a line from x1,y1 go x2,y2. Line can be drawn in any direction.    */
/****************************************************************************/
void GLCD_Line(int x1, int y1, int x2, int y2, drawmode show) 
{
  int dy = y2 - y1;
  int dx = x2 - x1;
  int stepx, stepy, fraction;
  if (dy < 0) 
  {
    dy = -dy;
    stepy = -1;
  }
  else 
  {
    stepy = 1;
  }
  if (dx < 0)
  {
    dx = -dx;
    stepx = -1;
  }
  else
  {
    stepx = 1;
  }
  dy <<= 1;
  dx <<= 1;
  GLCD_Pixel(x1,y1,show);
  if (dx > dy)
  {
    fraction = dy - (dx >> 1); 
    while (x1 != x2)
    {
      if (fraction >= 0)
      {
        y1 += stepy;
        fraction -= dx;
      }
      x1 += stepx;
      fraction += dy;  
      GLCD_Pixel(x1,y1,show);
    }
  }
  else
  {
    fraction = dx - (dy >> 1);
    while (y1 != y2)
    {
      if (fraction >= 0)
      {
        x1 += stepx;
        fraction -= dy;
      }
      y1 += stepy;
      fraction += dx;
      GLCD_Pixel(x1,y1,show);
    }
  }
}



/*********************************************************/
/* Print Char ( used for STDIO in avr-libc )             */
/*********************************************************/
int GLCD_PrintChar(char myChar)
{  
  GLCD_Char(myChar);
  return 0;
}

/**********************************************************/
/* Draw whole block of picture                            */
/**********************************************************/
void GLCD_DrawPicture(int x, int y, int width, int high, PGM_P pt)
{
   long  i, n_Counter;
   int   k,m;
   int   x_index, y_index;
   unsigned char  Counter, Str;
  
  
   i = n_Counter = 0;
   while(i < (width * high / 8 ) )
   {
		   Str = pgm_read_byte (&pt[n_Counter++]);
		   if((Str == 0x00) ||(Str == 0xff))
		          Counter = pgm_read_byte (&pt[n_Counter++]);
           else
	              Counter =  1;               
           for(  m = 0; m < Counter  ; ++m)
           {
           y_index = i / (width / 8);
           x_index = i % (width / 8);
			for(k = 0; k < 8; ++k)
			    {		    
				   if(Str & (1<<k))
					    GLCD_Pixel (x_index * 8 + k + x, y_index + y, on);  			
			    }
            ++i;
           }
	}          

}             

