/*****************************************************************************	
*									- C108.H -
*						Declarations used with the C108
*****************************************************************************/

#ifndef _TEST_LCD_INCLUDED
#define _TEST_LCD_INCLUDED
//
#include <stdio.h>
#include <stdbool.h>
#include <math.h>
#include <avr/io.h>
#include <avr/pgmspace.h>
#include <avr/signal.h>
#include <avr/interrupt.h>
/*****************************************************************************
*           				SPI Definitions
*/
#define SPI_PORT			PORTB
#define SPI_DDR             DDRB	//Data direction register
#define SPI_PIN             PINB	//Port used for SPI
#define SPI_SS            	PORTB4		//SPI Slave select, must be set as output
#define SPI_MOSI            PORTB5		//SPI CPU master output
#define SPI_MISO            PORTB6		//SPI CPU master input
#define SPI_SCK				PORTB7		//SPI clock, CPU master

/*****************************************************************************
*           				Serial EEprom Definitions
*/
#define SERIALEE_PAGENUM	(512)		//The number of pages in the serial EE
#define SERIALEE_PAGEOFFSET	(0)			//The number of pages not used for record storage
#define SERIALEE_PAGESIZE	(64)		//The number of bytes in a page of the serial EE

/*****************************************************************************
*           				Definitions
*/
#define CLOCKFREQ	  				(7372800)	//System clock frequency
#define DISPLAY_LENGTH				(20)
#define TEMPSTRSIZE 				(128)		//Size of the temp string
#define MAX_CMD_LEN 				(64)		//Max input cmd length, 1 char for termination

//
#define TAB 						(0x09)
#define BACKSPACE 					(0x08)
#define SPACE 						(0x20)
#define PERIOD 						(0x2E)
#define CR 		    				(0x0D)
#define LF 		    				(0x0A)
#define ESCAPE	    				(0x1B)
//
#define FALSE 						(0x00)
#define TRUE 						(0x01)

#define   Picture_Width   104
#define   Picture_High   125

//Store Graphic into Flash
//Generate By ?????????????.EXE: Used Zip option
const char RSTLOGO[]PROGMEM = 
{0x00,0x0B,0xCC,0x08,0x00,0x0B,0xCF,0x1C,0x00,0x0B,0x73,0x17,0x00,0x0A,0x80,0x31,
0x03,0x00,0x0A,0x80,0xDE,0x0D,0x00,0x0B,0xC4,0x04,0x00,0x0A,0xC0,0x7F,0x07,0x00,
0x0A,0x80,0x31,0x07,0x00,0x0A,0xC0,0xFF,0x01,0x03,0x00,0x0A,0x40,0xCC,0x01,0x00,
0x0A,0x60,0xCE,0x01,0x00,0x0A,0xA0,0x33,0x03,0x00,0x0A,0x80,0x31,0x01,0xFF,0x0B,
0xCE,0x1F,0xFF,0x0A,0x6F,0xC4,0x1F,0xFF,0x0C,0x1F,0xFF,0x0A,0x9F,0x31,0x1F,0xFF,
0x0C,0x1F,0xFF,0x0A,0xE7,0xC4,0x1F,0xFF,0x0A,0x67,0xCE,0x1F,0xFF,0x0A,0x9F,0xF3,
0x1F,0xFF,0x0A,0x9F,0xF1,0x1F,0xFF,0x0A,0xE7,0xEE,0x1F,0xFF,0x0A,0x63,0xE4,0x1F,
0xFF,0x0C,0x1F,0xFF,0x0A,0x99,0xF1,0x1F,0xFF,0x0A,0xFD,0xFF,0x01,0x1F,0xFF,0x0A,
0x67,0xFE,0x1F,0xFF,0x0A,0x67,0xFE,0x1F,0xFF,0x0A,0xB9,0xFB,0x1F,0xFF,0x0A,0x98,
0xF9,0x1F,0xFF,0x09,0x7F,0x6F,0xFE,0x1F,0xFF,0x09,0x7F,0x66,0xFC,0x1F,0xFF,0x0C,
0x1F,0xFF,0x09,0x3F,0x00,0x01,0xFC,0x1F,0xFF,0x09,0x3F,0x00,0x01,0xFE,0x1F,0xFF,
0x09,0x1F,0x00,0x01,0xFE,0x1F,0xFF,0x09,0x1F,0x00,0x01,0xFF,0x01,0x1F,0xFF,0x09,
0x1F,0x00,0x01,0xFF,0x01,0x1F,0xFF,0x09,0x0F,0x80,0xFF,0x01,0x1F,0xFF,0x09,0x0F,
0x80,0xFF,0x01,0x1F,0xFF,0x09,0x0F,0xC0,0xFF,0x01,0x1F,0xFF,0x09,0x07,0xC0,0xFF,
0x01,0x1F,0xFF,0x09,0x07,0xC0,0xFF,0x01,0x1F,0xFF,0x04,0xF9,0xFF,0x01,0x1F,0xFC,
0xFF,0x01,0x03,0xE0,0xFF,0x01,0x1F,0xFF,0x02,0x00,0x01,0x7C,0xE0,0x7F,0x01,0xC0,
0x1F,0x00,0x02,0xF0,0x1F,0xFF,0x01,0x7F,0x00,0x01,0x1C,0xF0,0x5F,0x00,0x02,0x1F,
0x00,0x02,0xF8,0x1F,0xFF,0x02,0x00,0x01,0x0C,0xF0,0x0F,0x00,0x02,0x1F,0x00,0x02,
0xF8,0x1F,0xFF,0x01,0x7F,0x00,0x01,0x04,0xF8,0x03,0x00,0x02,0x0F,0x00,0x02,0xFC,
0x1F,0xFF,0x01,0x7F,0x00,0x01,0x04,0xF8,0x03,0x00,0x01,0x80,0x0F,0x00,0x02,0xFC,
0x1F,0xFF,0x01,0x3F,0x00,0x02,0xFC,0x00,0x02,0x80,0x0F,0x00,0x02,0xFE,0x1F,0xFF,
0x01,0x3F,0x00,0x02,0x7C,0x00,0x02,0x80,0x07,0x00,0x02,0xFE,0x1F,0xFF,0x01,0x3F,
0x00,0x02,0x3C,0x00,0x02,0xC0,0x07,0x00,0x02,0xFE,0x1F,0xFF,0x01,0x1F,0x00,0x02,
0x3C,0x00,0x01,0xF0,0xC0,0x77,0x00,0x01,0xF8,0xFF,0x01,0x1F,0xFF,0x01,0x0F,0x00,
0x02,0x1C,0x00,0x01,0xFC,0xC3,0x7F,0x00,0x01,0xF8,0xFF,0x01,0x1F,0xFF,0x01,0x1F,
0x00,0x01,0x08,0x1C,0x00,0x01,0xFE,0xCF,0x3F,0x00,0x01,0xFC,0xFF,0x01,0x1F,0xFF,
0x01,0x0F,0x00,0x01,0xFE,0x1F,0x00,0x01,0xFE,0xFF,0x01,0x7F,0x00,0x01,0xFC,0xFF,
0x01,0x1F,0xFF,0x01,0x0F,0x00,0x01,0xFF,0x01,0x1F,0x00,0x01,0xFE,0xFF,0x01,0x3F,
0x00,0x01,0xFE,0xFF,0x01,0x1F,0xFF,0x01,0x07,0x80,0xFF,0x01,0x1F,0x00,0x01,0xFC,
0xFF,0x01,0x3F,0x00,0x01,0xFE,0xFF,0x01,0x1F,0xFF,0x01,0x07,0x80,0xFF,0x01,0x1F,
0x00,0x01,0xC0,0xFF,0x01,0x1F,0x00,0x01,0xFF,0x02,0x1F,0xFF,0x01,0x07,0x80,0xFF,
0x01,0x1F,0x00,0x01,0x80,0xFF,0x01,0x1F,0x00,0x01,0xFF,0x02,0x1F,0xFF,0x01,0x03,
0xC0,0xFF,0x01,0x1F,0x00,0x02,0xFE,0x1F,0x00,0x01,0xFF,0x02,0x1F,0xFF,0x01,0x03,
0xC0,0xFF,0x01,0x3F,0x00,0x02,0xFC,0x0F,0x00,0x01,0xFF,0x02,0x1F,0xFF,0x01,0x01,
0xC0,0xFF,0x01,0x3F,0x00,0x02,0xF8,0x0F,0x00,0x01,0xFF,0x02,0x1F,0xFF,0x01,0x03,
0xE0,0xFF,0x02,0x00,0x02,0xF8,0x07,0x80,0xFF,0x02,0x1F,0xFF,0x01,0x01,0xE0,0xFF,
0x02,0x03,0x00,0x01,0xF8,0x0F,0x80,0xFF,0x02,0x1F,0xFF,0x01,0x01,0xE0,0xFF,0x02,
0x0F,0x00,0x01,0xF8,0x07,0xC0,0xFF,0x02,0x1F,0xFF,0x01,0x00,0x01,0xF0,0xFF,0x02,
0x3F,0x00,0x01,0xF8,0x07,0xC0,0xFF,0x02,0x1F,0xFF,0x01,0x00,0x01,0xF0,0xFF,0x02,
0x7F,0x00,0x01,0xF8,0x03,0xE0,0xFF,0x02,0x1F,0xFF,0x01,0x00,0x01,0xF8,0xFF,0x01,
0xFE,0x3F,0x00,0x01,0xF8,0x03,0xE0,0xFF,0x02,0x1F,0x7F,0x00,0x01,0xF8,0xFF,0x01,
0xF8,0x3F,0x00,0x01,0xF8,0x03,0xE0,0xFF,0x02,0x1F,0x7F,0x00,0x01,0xF8,0xFF,0x01,
0xF0,0x3F,0x00,0x01,0xFC,0x01,0xE0,0xFF,0x02,0x1F,0x3F,0x00,0x01,0xF8,0x7F,0xE0,
0x17,0x00,0x01,0xFC,0x01,0xC0,0xFF,0x02,0x1F,0x3F,0x00,0x01,0xFC,0x7F,0x00,0x01,
0x03,0x00,0x01,0xFE,0x00,0x02,0xFC,0xFF,0x01,0x1F,0x1F,0x00,0x01,0xFE,0x7F,0x00,
0x03,0xFE,0x01,0x00,0x01,0xFE,0xFF,0x01,0x1F,0x3F,0x00,0x01,0xFE,0x7F,0x00,0x02,
0x80,0xFF,0x01,0x01,0x00,0x01,0xFE,0xFF,0x01,0x1F,0x1F,0x00,0x01,0xFE,0x3F,0x00,
0x02,0xC0,0xFF,0x01,0x01,0x00,0x01,0xFE,0xFF,0x01,0x1F,0x1F,0x00,0x01,0xFF,0x01,
0x3F,0x00,0x02,0xC0,0xFF,0x01,0x00,0x02,0xFF,0x02,0x1F,0x0F,0x00,0x01,0xFF,0x01,
0x1F,0x00,0x02,0xF0,0xFF,0x01,0x00,0x02,0xFF,0x02,0x1F,0x0F,0x80,0xFF,0x01,0x1F,
0x00,0x02,0xF8,0x7F,0x02,0x00,0x01,0xFF,0x02,0x1F,0x07,0x80,0xFF,0x02,0x01,0x80,
0xFF,0x01,0x7F,0x03,0x80,0xFF,0x02,0x1F,0xFF,0x04,0x07,0xE0,0xFF,0x02,0x09,0xE0,
0xFF,0x02,0x1F,0xFF,0x08,0x98,0xFF,0x03,0x1F,0xFF,0x07,0x3F,0xEF,0xFE,0xFF,0x02,
0x1F,0xFF,0x07,0x3F,0x66,0xFC,0xFF,0x02,0x1F,0xDF,0xBF,0x48,0x49,0x9B,0x3F,0xCE,
0xDF,0xBF,0xFF,0x03,0x1F,0x4F,0x89,0x48,0x61,0x89,0x25,0xC5,0x8F,0x98,0xFF,0x03,
0x1F,0x6F,0xEC,0x4B,0x25,0xED,0x70,0xF7,0xFF,0x05,0x1F,0x2F,0xCC,0x8D,0x25,0xE6,
0xB0,0xF7,0x37,0x66,0xFE,0xFF,0x02,0x1F,0xA7,0xDC,0xAD,0xB4,0xF5,0xBA,0xE7,0xF7,
0x7F,0xFF,0x03,0x1F,0xB7,0xCE,0xA5,0xD4,0x75,0xBA,0xEF,0xCF,0x99,0xFF,0x03,0x10,
0xB3,0xE2,0x96,0xD8,0xC2,0x9A,0xF1,0xCF,0x18,0xFE,0x7F,0x00,0x01,0x10,0xFF,0x07,
0x73,0xE7,0x80,0x05,0xF8,0x1F,0xFF,0x07,0x33,0xE6,0x0F,0xE0,0xFF,0x01,0x1F,0xFF,
0x07,0xDF,0xBF,0x7F,0xFC,0xFF,0x01,0x1F,0xFF,0x06,0x9F,0xCD,0x98,0xFF,0x03,0x1F,
0xFF,0x06,0x07,0xFF,0x05,0x1F,0xFF,0x06,0xC3,0x33,0xE6,0xFF,0x03,0x1F,0xFF,0x02,
0x3F,0x00,0x02,0x80,0xF8,0x3B,0xE7,0xFF,0x03,0x1F,0xFF,0x02,0x47,0xFE,0xFF,0x01,
0x0F,0xFC,0xDC,0xF9,0xFF,0x03,0x1F,0xFF,0x01,0x00,0x01,0xE1,0xFF,0x02,0x7F,0xFF,
0x01,0xCC,0xF8,0xFF,0x03,0x1F,0x1F,0xFE,0xFC,0xFF,0x04,0x73,0xF7,0xFF,0x03,0x1F,
0x87,0xFF,0x06,0x33,0xF2,0xFF,0x03,0x1F,0xF1,0xFF,0x06,0xDE,0xFF,0x04,0x1F,0xFF,
0x06,0x7F,0x8C,0xF8,0xFF,0x03,0x1F,0xFF,0x0C,0x1F,0xFF,0x06,0x3F,0x71,0xFE,0xFF,
0x03,0x1F,0xFF,0x07,0xF3,0xFF,0x04,0x1F,0xFF,0x07,0xCC,0xFC,0xFF,0x03,0x1F,0xFF,
0x06,0x7F,0xCC,0xFF,0x04,0x1F,0xFF,0x06,0x9F,0x77,0xFF,0x04,0x1F,0xFF,0x06,0x1F,
0x31,0xFE,0xFF,0x03,0x1F,0xFF,0x06,0xEF,0xDF,0xFF,0x04,0x1F,0xFF,0x06,0x4F,0xCC,
0xFF,0x04,0x1F,0x00,0x06,0xF8,0xFF,0x01,0x00,0x0B,0x38,0x33,0x00,0x0B,0xBC,0x73,
0x00,0x0B,0xE6,0xDC,0x00,0x0B,0x66,0x4C,0x00,0x0B,0xBC,0x37,0x00,0x0B,0x18,0x33,
0x00,0x0B,0xFE,0x3F,0x00,0x0B,0x66,0x0C,0x00,0x0B,0xFF,0x01,0x3F,0x00,0x0A,0x80,
0x19,0x13,0x00,0x05,
};

#endif
